"""
Project: kiltera 
Module: channels
Author: Ernesto Posse
Created on: Jun 9, 2006
Last modified: Jan 22, 2007

Description:
  This implements channels for communication between Aspa processes.
"""

from values import *
from threading import *
from kthreads import *
from posse_tools.utils import log
import pickle

class BaseChannel(Value):
    
    counter = 0
    
    def __init__(self):
        self.data = None
        BaseChannel.counter += 1
        self.id = BaseChannel.counter
        
    def send(self, data=None):
        pass
        
    def receive(self):
        return None
    
    def pyvalue(self):
        return self


class Buffer(BaseChannel):
    
    counter = 0
    
    def __init__(self,name=None,loc=None, thread=None):
        BaseChannel.__init__(self)
        self.data = []
        self.condition_lock = RLock(verbose=False)
        self.condition = Condition(self.condition_lock,verbose=False)
        self.value = self.data
        self.name = name
        self.loc = loc
        self.thread = thread
        #~ print "BUFF THREAD",thread
        #~ if isinstance(thread, KThread):
            #~ self.thread.add_condvar(self.condition)
        Buffer.counter += 1
        self.id = Buffer.counter
       
    def send(self, data=None):
        #~ print "send 1",self,self.data,self.condition,id(self.condition)
        self.condition.acquire()
        #~ print "send 2",self,self.data,self.condition,id(self.condition)
        self.data.append(data)
        #~ print "send 3",self,self.data,self.condition,id(self.condition)
        self.value = self.data
        self.condition.notifyAll()
        #~ print "send 4",self,self.data,self.condition,id(self.condition)
        self.condition.release()
        #~ print "send 5",self,self.data,self.condition,id(self.condition)
        
    def receive(self):
        #~ print "* recv 1",self,self.data,self.condition,id(self.condition)
        self.condition.acquire()
        #~ print "* recv 1.2",self,self.data,self.condition,id(self.condition)
        while not self.data_available():
            #~ print "* recv 1.3",self,self.data,self.condition,id(self.condition)
            #~ print "---1",currentThread()
            if currentThread().killed:
                #~ print "### 1",currentThread()
                self.condition.release()
                raise SystemExit()
            self.condition.wait()
            #~ print "---2",currentThread()
            if currentThread().killed:
                print "### 2",currentThread(),self.condition,self.data
                self.condition.notifyAll()
                self.condition.release()
                raise SystemExit()
            #~ print "* recv 1.4",self,self.data,self.condition,id(self.condition)
        if currentThread().killed:
            #~ print "### 3"
            self.condition.release()
            raise SystemExit()
        #~ print "* recv 2",self,self.data,self.condition,id(self.condition)
        first = self.data[0]
        self.data = self.data[1:]
        self.value = self.data
        self.condition.release()
        return first
        
    def data_available(self):
        return len(self.data) > 0
        
    def reset(self):
        self.data = []
    
    def wakeup(self):
        #~ print "* waking up 1",self.condition,id(self.condition)
        self.condition.acquire()
        #~ print "* waking up 2",self.condition,id(self.condition)
        self.condition.notifyAll()
        #~ print "* waking up 3",self.condition,id(self.condition)
        self.condition.release()
        #~ print "* waking up 4",self.condition,id(self.condition)
        
    def __str__(self):
        #~ return str(self.data)
        #~ return str([str(datum) for datum in self.data])
        #~ print '***',type(self.loc),self.loc
        if not self.loc:
            s = "{chan " + "?." + str(self.name) 
        else:
            s = "{chan " + '.'.join([c.name for c in self.loc]) + "." + self.name  
        s += ';id:'+str(self.id)
        s += ';contents:' + str([str(datum) for datum in self.data]) + "}"
        return s

    def __repr__(self):
        return repr([repr(datum) for datum in self.data])



AsyncUnicastChannel = Buffer

class SyncUnicastChannel(BaseChannel):
    
    def __init__(self):
        BaseChannel.__init__(self)
        self.data = None
        self.sender_lock    = KBinarySemaphore('sender_lock',True)
        self.receiver_lock  = KBinarySemaphore('receiver_lock',True)
        self.sender_ready   = KBinarySemaphore('sender_ready',False)
        self.receiver_ready = KBinarySemaphore('receiver_ready',False)
        self.commit = False
        #~ self.commit_mutex = KBinarySemaphore('mutex',True)
        self.confirmation = KBinarySemaphore('confirm',False)
    
    def __repr__(self):
        return "%s(%d,%s,%s,%s,%s,%s)" % (self.__class__.__name__, self.id, self.sender_lock, self.receiver_lock, self.sender_ready, self.receiver_ready, self.confirmation)
        
    def send(self, data):
        log.note(3, "1")
        self.sender_lock.acquire()
        log.note(3, "2")
        self.data = data
        self.sender_ready.release()
        log.note(3, "3")
        self.receiver_ready.acquire(external_lock=self.sender_lock, 
                                    reset_sems=[(self.sender_ready,False), (self.confirmation,False)]) 
                                    #~ release_sems=[self.confirmation])
        log.note(3, "4")
        self.confirmation.release()
        log.note(3, "5")
        self.sender_lock.release()
        log.note(3, "6")
    
    def receive(self):
        log.note(3, "1")
        self.receiver_lock.acquire()
        log.note(3, "2")
        self.sender_ready.acquire(external_lock=self.receiver_lock,
                                  reset_sems=[(self.receiver_ready,False), (self.confirmation,False)], 
                                  release_sems=[]) #self.confirmation])
        log.note(3, "3")
        received = self.data
        #~ sleep(2)
        self.receiver_ready.release()
        log.note(3, "4")
        #~ sleep(2)
        self.confirmation.acquire(external_lock=self.receiver_lock,
                                  reset_sems=[(self.receiver_ready,False), (self.confirmation,False)],
                                  release_sems=[self.sender_ready])
        log.note(3, "5")
        received = self.data
        log.note(3, "6")
        self.receiver_lock.release()
        log.note(3, "7")
        return received

    #~ def receive(self):
        #~ log.note(3, "1")
        #~ self.receiver_lock.acquire()
        #~ log.note(3, "2")
        #~ done = False
        #~ while not done:
            #~ log.note(3, "3")
            #~ self.sender_ready.acquire(external_lock=self.receiver_lock,
                                      #~ reset_sems=[(self.receiver_ready,False), (self.confirmation,False)], 
                                      #~ release_sems=[]) #self.confirmation])
            #~ log.note(3, "4")
            #~ received = self.data
            #~ self.receiver_ready.release()
            #~ log.note(3, "5")
            #~ self.confirmation.acquire(external_lock=self.receiver_lock,
                                      #~ reset_sems=[(self.receiver_ready,False), (self.confirmation,False)],
                                      #~ release_sems=[self.sender_ready])
            #~ log.note(3, "6")
            #~ if self.commit:
                #~ received = self.data
                #~ done = True
                #~ log.note(3, "7")
            #~ else:
                #~ received = None
                #~ log.note(3, "8")
        #~ self.commit = False
        #~ self.receiver_lock.release()
        #~ log.note(3, "9")
        #~ return received

    def wakeup(self):
        self.wakeup_senders()
        self.wakeup_receivers()
    
    def wakeup_senders(self):
        self.receiver_ready.wakeup()
        self.sender_lock.wakeup()

    def wakeup_receivers(self):
        self.sender_ready.wakeup()
        self.receiver_lock.wakeup()
        self.confirmation.wakeup()


class RemoteAsyncChannel(BaseChannel):

    def __init__(self, socket, bufsize=2**16):
        self.socket = socket
        self.bufsize = bufsize
        self.data = None
        
    def send(self, data=None):
        log.note(3, "1 -- %s -- %s -- %s" % (currentThread(), repr(data),self.socket))
        msg = pickle.dumps(data.pyvalue())
        self.socket.send(msg)
        log.note(3, "2 -- %s -- %s" % (currentThread(), repr(data)))
        
    def receive(self):
        log.note(3, "1 -- %s" % (currentThread()))
        data = self.socket.recv(self.bufsize)
        log.note(3, "2 -- %s -- %s" % (currentThread(), repr(data)))
        unpacked = pickle.loads(data)
        klt_data = PyValue(unpacked).klt_value     # Note: I need to make it into a kiltera value again!
        log.note(3, "3 -- %s -- %s" % (currentThread(), repr(unpacked)))
        return klt_data
    
    def wakeup(self):
        log.note(3, "1 -- %s" % (currentThread()))
    
    def pyvalue(self):
        return self


class RemoteSyncChannel(BaseChannel):

    def __init__(self, socket, bufsize=2**16):
        self.socket = socket
        self.bufsize = bufsize
        self.data = None


class SyncMulticastChannel(BaseChannel):
    pass


class AsyncMulticastChannel(BaseChannel):
    pass


shorthands = \
    {
        'au': AsyncUnicastChannel,
        'su': SyncUnicastChannel,
        'am': AsyncMulticastChannel,
        'sm': SyncMulticastChannel,
        'ra': RemoteAsyncChannel,
        'rs': RemoteSyncChannel
    }


def Channel(channel_type='au'):
    return shorthands[channel_type]()


AsyncChannel = Buffer
SyncChannel  = SyncUnicastChannel
