"""
Project: kiltera 
Module: ktime
Author: Ernesto Posse
Created on: Sep 8, 2006
Last modified: Sep 8, 2006

Description:
  This module implements logical and physical clocks for kiltera.
"""

import time

class LogicalClock:
  
    def __init__(self, initial_time = 0.0):
        self.__time = initial_time
    
    def get_time(self):
        return self.__time
    
    def set_time(self, t):
        self.__time = t
    
    time = property(get_time, set_time)
    
    def delay(self, dt):
        self.__time += dt
        
    sleep = delay


class PhysicalClock:
  
    def __init__(self, initial_time = 0.0):
        self.__time = initial_time
        self.__init_time = time.time()

    def get_time(self):
        return time.time() - self.__init_time
    
    time = property(get_time)
    
    def delay(self, dt):
        time.sleep(dt)
    
    sleep = delay



