"""
Project: kiltera 
Module: channels
Author: Ernesto Posse
Created on: Sep 25, 2006
Last modified: Sep 25, 2006

Description:
  This implements simulated channels for communication between kiltera processes.
"""

from values import *
from random import randint

FIRST_COME_FIRST_SERVED = 0
RANDOM = 1

class BaseChannel(Value):
    
    def __init__(self):
        self.data = None
        
    def send(self, data=None):
        pass
        
    def receive(self):
        return None
    
    def pyvalue(self):
        return self


class Buffer(BaseChannel):
    
    def __init__(self, policy=RANDOM):
        BaseChannel.__init__(self)
        self.data = []
        self.value = self.data
        self.receivers = []
        self.policy = policy
       
    def send(self, data=None):
        self.data.append(data)
        self.value = self.data
        
    def receive(self):
        try:
            first = self.data[0]
            del self.data[0]
            self.value = self.data
            return first
        except IndexError:
            raise EmptyChannel
        
    def data_available(self):
        return len(self.data) > 0
        
    def __str__(self):
        return str([str(datum) for datum in self.data])


AsyncUnicastChannel = Buffer


class SyncUnicastChannel(BaseChannel):
    
    def __init__(self):
        BaseChannel.__init__(self)
        self.transport = Buffer()
        #~ self.ack = Buffer()
        
    def send(self, data=None):
        private1 = Buffer()
        self.transport.send(private1)
        private2 = private1.receive()
        private2.send(data)
        
    def receive(self):
        private1 = self.transport.receive()
        private2 = Buffer()
        private1.send(private2)
        data = private2.receive()
        return data
        
    def __str__(self):
        return str(self.transport)


def Channel(channel_type='au'):
    if channel_type is 'su':
        return SyncUnicastChannel()
    elif channel_type is 'au':
        return AsyncUnicastChannel()
    elif channel_type is 'sm':
        return SyncMulticastChannel()
    elif channel_type is 'am':
        return AsyncMulticastChannel()

AsyncChannel = Buffer
SyncChannel  = SyncUnicastChannel


class EmptyChannel(Exception):
    pass
    