"""
Project: kiltera 
Module: visitor
Author: Ernesto Posse
Created on: Jun 9, 2006
Last modified: Dec 14, 2006

Description:
  This module provides the interface for visitors to use the visitor design pattern.
"""

class Visitor:
    
    # Atom visitors
    
    def visit_atom(self, atom, *args, **kwargs):
        pass
    def visit_output(self, output, *args, **kwargs):
        pass
    def visit_input(self, input, *args, **kwargs):
        pass
    def visit_assign(self, assign, *args, **kwargs):
        pass
    def visit_print(self, print_atom, *args, **kwargs):
        pass
    
    # Process visitors
    
    def visit_process(self, process, *args, **kwargs):
        pass
    def visit_nil(self, nil, *args, **kwargs):
        pass
    def visit_done(self, nil, *args, **kwargs):
        pass
    def visit_prefix(self, prefix, *args, **kwargs):
        pass
    def visit_send(self, send, *args, **kwargs):
        pass
    def visit_receive(self, receive, *args, **kwargs):
        pass
    def visit_parallel(self, parallel, *args, **kwargs):
        pass
    def visit_sequence(self, sequence, *args, **kwargs):
        pass
    def visit_local(self, local, *args, **kwargs):
        pass
    def visit_chanarray(self, chanarray, *args, **kwargs):
        pass
    def visit_let(self, let, *args, **kwargs):
        pass
    def visit_conditional(self, cond, *args, **kwargs):
        pass
    def visit_match(self, match_proc, *args, **kwargs):
        pass
    def visit_procinst(self, process_inst, *args, **kwargs):
        pass
    def visit_localdefs(self, localdefs, *args, **kwargs):
        pass
    def visit_indexedpar(self, indexedpar, *args, **kwargs):
        pass
    def visit_loop(self, loop, *args, **kwargs):
        pass
    def visit_interrupt(self, interrupt, *args, **kwargs):
        pass
    def visit_wait(self, wait, *args, **kwargs):
        pass
    def visit_timeout(self, timeout, *args, **kwargs):
        pass
    def visit_sendat(self, sendat, *args, **kwargs):
        pass
    def visit_receiveat(self, receiveat, *args, **kwargs):
        pass
        
    # Toplevel modules
    
    def visit_funcdef(self, funcdef, *args, **kwargs):
        pass
        
    def visit_processdef(self, processdef, *args, **kwargs):
        pass
    
    def visit_module(self, module, *args, **kwargs):
        pass

    # Expressions
    
    def visit_bottom(self, bottom, *args, **kwargs):
        pass

    def visit_variable(self, var, *args, **kwargs):
        pass
    
    def visit_boolconst(self, const, *args, **kwargs):
        pass
    
    def visit_compare(self, comp, *args, **kwargs):
        pass
        
    def visit_boolbinexpr(self, expr, *args, **kwargs):
        pass
    
    def visit_boolunexpr(self, expr, *args, **kwargs):
        pass
    
    def visit_arithconst(self, const, *args, **kwargs):
        pass
    
    def visit_arithbinexpr(self, expr, *args, **kwargs):
        pass
    
    def visit_arithunexpr(self, expr, *args, **kwargs):
        pass
    
    def visit_string(self, string, *args, **kwargs):
        pass
    
    def visit_tuple(self, tuple_expr, *args, **kwargs):
        pass
    
    def visit_cons(self, cons_expr, *args, **kwargs):
        pass
    
    def visit_comprehension(self, comprehension, *args, **kwargs):
        pass
    
    def visit_tupleaccess(self, tupleaccess_expr, *args, **kwargs):
        pass
    
    def visit_funccall(self, funccall, *args, **kwargs):
        pass
    
    def visit_newchan(self, newchan, *args, **kwargs):
        pass

    def visit_case(self, case_expr, *args, **kwargs):
        pass

    def visit_condexpr(self, cond_expr, *args, **kwargs):
        pass

    def visit_letexpr(self, let_expr, *args, **kwargs):
        pass

    # Builtins
    
    def visit_builtin(self, builtin, *args, **kwargs):
        pass
        

    